<?php
	
	/*
	 * This endpoint can receive single as well as multiple posts with their meta data.
	 * Function for this endpoint is to assign meta data to a post
	 * For multiple posts send a parameter named "meta_data" as [{"id":1,"meta":{"key1":"value1","key2":"value2"}},{"id":2,"meta":{"key1":"value1","key2":"value2"}},{"id":3,"meta":{"key1":"value1","key2":"value2"}}]
	 *
	 * For single send  parameters named "id"(post_id) and "data" like {"key1":"value1","key2":"value2"}
	 */
	class meta_post extends WC_API_Resource {
		
		protected $base = '/meta_post';
		
		public function register_routes( $routes ) {
			# GET /getcategorynsids
			$routes[ $this->base ] = [
				[
					[ $this, 'get_meta_post' ],
					WC_API_Server::READABLE | WC_API_SERVER::EDITABLE | WC_API_SERVER::ACCEPT_DATA,
				],
			];
			
			return $routes;
		}
		
		public function get_meta_post() {
			
			if ( ! empty( $_REQUEST["meta_data"] ) ) {
				$request = stripslashes( base64_decode( $_REQUEST["meta_data"] ) );
			} elseif ( ! empty( $_REQUEST["data"] ) && ! empty( $_REQUEST["id"] ) ) {
				//this will handle single post meta data, this is for older version
				$this->_get_meta_post_single();
				
				return;
			} else {
				echo json_encode( [ "errors" => "Request data is empty" ] );
				exit;
			}
			
			$data = json_decode( $request, TRUE );
			
			if ( empty( $data ) ) {
				echo json_encode( [ "errors" => "Posted json is invalid" ] );
				exit;
			}
			
			foreach ( $data as $value ) {
				
				$id   = $value['id'];
				$meta = $value['meta'];
				
				if ( ! empty( $id ) && ! empty( $meta ) && is_array( $meta ) ) {
					do_action( 'in8_endpoint_meta_post_before_update', $meta, $id );
					$meta = apply_filters('in8_endpoint_meta_post_data', $meta, $id);
					
					// Using this to keep track of the bundle items that we found, and going to remove the ones that are extra on wc
					$bundle_products_from_ns = [];
					
					foreach ( $meta as $key => $val ) {
						if ( apply_filters( 'in8_endpoint_meta_post_data_skip', FALSE, $id, $key, $val ) ) {
							continue;
						}
						
						// 04/20/2016 - If there is a custom field named "_bundle_data", set the product type to bundle
						// because the standard API doesn't allow to set the type to bundle
						if ( $key == "_bundle_data" && class_exists( 'WC_PB_DB' ) ) {
							wp_set_object_terms( $id, 'bundle', 'product_type' );
							
							// Gets the existing bundle items
							$bundle_results = WC_PB_DB::query_bundled_items( [
//								'return'    => 'ids',
								'bundle_id' => [ $id ],
							] );
							
							// Update bundle items
							foreach ( $val as $product_id => $bundle_product ) {
								
								$bundle_product_updated = FALSE;
								$DATA                   = [
									'bundle_id'  => $id,
									'product_id' => $product_id,
									'meta_data'  => $bundle_product,
								];
								
								
								foreach ( $bundle_results as $item ) {
									if ( $product_id == $item['product_id'] ) {
										$bundleRet              = WC_PB_DB::update_bundled_item( $item['bundled_item_id'], $DATA );
										$bundle_product_updated = TRUE;
									}
								}
								if ( ! $bundle_product_updated ) {
									$bundleRet = WC_PB_DB::add_bundled_item( $DATA );
								}
								
								$bundle_products_from_ns[] = $product_id;
							}
							
						}
						if ( $key == "_product_type" ) {
							$classname = WC_Product_Factory::get_classname_from_product_type( $val );
							
							if ( ! class_exists( $classname ) ) {
								$classname = 'WC_Product_Simple';
							}
							
							$product = new $classname( $id );
							$product->save();
						}
						if ( $key == "orderdate" ) {
							$date = strtotime( $val );
							wp_update_post( [
								'ID'        => $id,
								'post_date' => date( 'Y-m-d 00:00:00', $date ),
							] );
						}
						
						// Endpoint allows to change the post status
						if ( $key == "_post_status" ) {
							$post = [ 'ID' => $id, 'post_status' => $val ];
							wp_update_post( $post );
						}
						
						if ( $key == "_menu_order" ) {
							$post = [ 'ID' => $id, 'menu_order' => $val ];
							wp_update_post( $post );
						}
						
						$val = apply_filters('in8_endpoint_meta_post_data_field', $val, $key, $id);
						
						if ( is_array( $val ) ) {
							update_post_meta( $id, $key, $val );
						} else {
							update_post_meta( $id, $key, htmlspecialchars_decode( $val ) );
						}
						
					}
					
					if ( ! empty( $bundle_results ) && ! empty( $bundle_products_from_ns ) ) {
						foreach ( $bundle_results as $i ) {
							if ( ! in_array( $i['product_id'], $bundle_products_from_ns ) ) {
								WC_PB_DB::delete_bundled_item( $i['bundled_item_id'] );
							}
						}
					}
					// Collect all the images
					$alt_images    = []; $alt_images_empty = false;
					$image_main    = NULL;
					$image_gallery = []; $image_gallery_empty = false;
					foreach ( $meta as $key => $val ) {
						if ( empty( $val ) || ! wp_http_validate_url( $val ) ) {
							
							if( preg_match( '/alt_image_/i', $key ) ){
								$alt_images_empty = true;
							}
							if ( preg_match( '/image_gallery/i', $key ) ) {
								$image_gallery_empty = true;
							}
							if ( $key == 'image_main' ) {
								$image_main = 'empty';
							}
							
							continue;
						}
						
						if ( preg_match( '/alt_image_/i', $key ) ) {
							$alt_images[] = $this->upload_attachment_to_product( $val, $id );
						}
						if ( $key == 'image_main' ) {
							$image_main = $this->upload_attachment_to_product( $val, $id );
						}
						if ( preg_match( '/image_gallery/i', $key ) ) {
							$image_gallery[] = $this->upload_attachment_to_product( $val, $id );
						}
					}
					
					// Save the alternate images
					if ( ! empty( $alt_images ) || $alt_images_empty == true ) {
						$existing_alt_images = get_post_meta( $id, apply_filters('in8_product_additional_variation_images', '_wc_additional_variation_images'), TRUE );
						if ( ! empty( $existing_alt_images ) ) {
							$existing_alt_images = explode( ',', $existing_alt_images );
							foreach ( $existing_alt_images as $img ) {
								wp_delete_attachment( $img, TRUE );
//								wp_schedule_single_event(time()+ 15, 'wp_delete_attachment', [$img, true]);
							}
						}
						
						// Save the new images
						update_post_meta( $id, apply_filters('in8_product_additional_variation_images', '_wc_additional_variation_images'), implode( ',', $alt_images ) );
					}
					
					// Save the main product image
					if ( ! empty( $image_main ) ) {
						$existing_image_main = get_post_thumbnail_id( $id );
						if ( ! empty( $existing_image_main ) ) {
							wp_delete_attachment( $existing_image_main, TRUE );
//							wp_schedule_single_event(time()+ 15, 'wp_delete_attachment', [$existing_image_main, true]);
						}
						
						if( $image_main != 'empty' ) {
							set_post_thumbnail( $id, $image_main );
						}
					}
					
					// Save the gallery images
					if ( ! empty( $image_gallery ) || $image_gallery_empty == true ) {
						$existing_gallery_images = get_post_meta( $id, '_product_image_gallery', TRUE );
						if ( ! empty( $existing_gallery_images ) ) {
							$existing_gallery_images = explode( ',', $existing_gallery_images );
							foreach ( $existing_gallery_images as $img ) {
								wp_delete_attachment( $img, TRUE );
//								wp_schedule_single_event(time()+ 15, 'wp_delete_attachment', [$img, true]);
							}
						}
						
						// Save the new images
						update_post_meta( $id, '_product_image_gallery', implode( ',', $image_gallery ) );
					}
					
					do_action( 'in8_endpoint_meta_post_after_update', $meta, $id );
				}
			}
			
			echo json_encode( [ "success" => "true" ] );
			
			exit;
		}
		
		private function _get_meta_post_single() {
			
			// They want to update or add data
			
			echo "test:";
			
			print_r( $_REQUEST );
			
			$id = $_REQUEST["id"];
			
			$data = json_decode( base64_decode( $_REQUEST["data"] ), TRUE );
			
			echo "\n";
			echo "id = " . $id . "\n";
			echo "data = " . json_encode( $data ) . "\n";
			
			if ( ! isset( $id ) ) {
				echo json_encode( [ "errors" => "Missing id argument. POST" ] );
				
				exit;
			}
			
			if ( ! isset( $data ) ) {
				echo json_encode( [ "errors" => "Missing data argument." ] );
				
				exit;
			}
			do_action( 'in8_endpoint_meta_post_before_update', $data, $id );
			$data = apply_filters('in8_endpoint_meta_post_data', $data, $id);
			
			$orderstatus = NULL;
			
			// Using this to keep track of the bundle items that we found, and going to remove the ones that are extra on wc
			$bundle_products_from_ns = [];
			
			foreach ( $data as $key => $val ) {
				// If there is a custom field named "_bundle_data", set the product type to bundle
				// because the standard API doesn't allow to set the type to bundle
				if ( $key == "_bundle_data" && class_exists( 'WC_PB_DB' ) ) {
					wp_set_object_terms( $id, 'bundle', 'product_type' );
					
					// Gets the existing bundle items
					$bundle_results = WC_PB_DB::query_bundled_items( [
//								'return'    => 'ids',
						'bundle_id' => [ $id ],
					] );
					
					// Update bundle items
					foreach ( $val as $product_id => $bundle_product ) {
						
						$bundle_product_updated = FALSE;
						$DATA                   = [
							'bundle_id'  => $id,
							'product_id' => $product_id,
							'meta_data'  => $bundle_product,
						];
						
						
						foreach ( $bundle_results as $item ) {
							if ( $product_id == $item['product_id'] ) {
								$bundleRet              = WC_PB_DB::update_bundled_item( $item['bundled_item_id'], $DATA );
								$bundle_product_updated = TRUE;
							}
						}
						if ( ! $bundle_product_updated ) {
							$bundleRet = WC_PB_DB::add_bundled_item( $DATA );
						}
						
						$bundle_products_from_ns[] = $product_id;
					}
					
				}
				
				// If the key is currency default is CAD, assigns the product type
				if ( $key == "caddefault" ) {
					$currd = '{"CAD":"' . $val . '"}';
					update_post_meta( $id, '_regular_currency_prices', $currd );
					//update_post_meta($id, $key, $currd);
					continue;
				}
				
				// If the key is _product_type, assigns the product type
				if ( $key == "_product_type" ) {
					$classname = WC_Product_Factory::get_classname_from_product_type( $val );
					
					if ( ! class_exists( $classname ) ) {
						$classname = 'WC_Product_Simple';
					}
					
					$product = new $classname( $id );
					$product->save();
				}
				if ( $key == "orderdate" ) {
					$date = strtotime( $val );
					wp_update_post( [
						'ID'        => $id,
						'post_date' => date( 'Y-m-d 00:00:00', $date ),
					] );
				}
				
				if ( $key == "menu_order" ) {
					
					wp_update_post( [
						'ID'         => $id,
						'menu_order' => $val,
					] );
				}
				
				// Endpoint allows to change the post status
				if ( $key == "_post_status" ) {
					$post = [ 'ID' => $id, 'post_status' => $val ];
					wp_update_post( $post );
				}
				
				// Get the order status
				if ( $key == "status" ) {
					$orderstatus = $val;
				}
				
				// If the payment method is PayPal and it has the order status in the request it captures / refunds order
				// On the mappings in NetSuite, include the fields "status" and "paymentmethod"
				if ( $key == 'paymentmethod' && $val == 'PayPal' ) {
					$gateway = new WC_Gateway_Paypal();
					
					// Check the order status sent from NetSuite
					if ( $orderstatus == 'refunded' ) {
						// Refund order
						$gateway->process_refund( $id );
					} else if ( $orderstatus == 'completed' ) {
						// Capture the payment
						$gateway->capture_payment( $id );
					}
				}
				
				// Changes for the Wholesale data
				//if ($key == "net15" || $key == "net30" || $key == "net45" || $key == "net60" || $key == "wholesale_customer") {
				//if (substr($key, 0, 10) == 'wholesale_') {
				///Start for serialized array
				//meta key = $wholesale_role
				//meta_ value = $wholesale_role_price
				//$wholesale_role_prices = [
				//'net15' => 'net15_wholesale_price',
				//'net30' => 'net30_wholesale_price',
				//'net45' => 'net45_wholesale_price',
				//'net60' => 'net60_wholesale_price',
				//'wholesale_customer' => 'wholesale_customer_wholesale_price'
				//];
				
				//foreach ($wholesale_role_prices as $wholesale_role => $wholesale_role_price_key) {
				//	$wholesale_role_price = get_post_meta($post_id, $wholesale_role_price_key, TRUE);
				//    if (!empty($wholesale_role_price)){
				//        $whole_qty_discount_arry[] = [ 'wholesale_role' => $wholesale_role, 'start_qty' => 1, 'end_qty' => 4, 'price_type' => 'fixed-price', 'wholesale_price' => $wholesale_role_price ];
				//    }
				//}
				
				//$whole_qty_discount_arry = [];

//                    $whole_qty_discount_arry[] = [ 'wholesale_role' => $key . '_wholesale_price', 'start_qty' => 1, 'end_qty' => 4, 'price_type' => 'fixed-price', 'wholesale_price' => $val ];
//
//					$serialized_array = serialize($whole_qty_discount_arry);
//
//					$key = 'wwpp_post_meta_quantity_discount_rule_mapping';
//
//					update_post_meta($id, $key, $serialized_array);
//				}
				
				
				if ( $key == "wsvisible" ) {
					delete_post_meta( $id, 'wwpp_product_wholesale_visibility_filter' );
					
					if ( is_array( $val ) ) {
						
						foreach ( $val as $vals ) {
							add_post_meta( $id, 'wwpp_product_wholesale_visibility_filter', $vals );
						}
						
					} else {
						add_post_meta( $id, 'wwpp_product_wholesale_visibility_filter', $val );
					}
					
					//continue;
				}
				
				$val = apply_filters('in8_endpoint_meta_post_data_field', $val, $key, $id);
				if ( is_array( $val ) ) {
					update_post_meta( $id, $key, $val );
				} else {
					update_post_meta( $id, $key, htmlspecialchars_decode( $val ) );
				}
			}
			
			if ( ! empty( $bundle_results ) && ! empty( $bundle_products_from_ns ) ) {
				foreach ( $bundle_results as $i ) {
					if ( ! in_array( $i['product_id'], $bundle_products_from_ns ) ) {
						WC_PB_DB::delete_bundled_item( $i['bundled_item_id'] );
					}
				}
			}
			
			
			// Collect all the images
			$alt_images    = []; $alt_images_empty = false;
			$image_main    = NULL;
			$image_gallery = []; $image_gallery_empty = false;
			foreach ( $data as $key => $val ) {
				if ( empty( $val ) || ! wp_http_validate_url( $val ) ) {
					
					if( preg_match( '/alt_image_/i', $key ) ){
						$alt_images_empty = true;
					}
					if ( preg_match( '/image_gallery/i', $key ) ) {
						$image_gallery_empty = true;
					}
					if ( $key == 'image_main' ) {
						$image_main = 'empty';
					}
					
					continue;
				}
				
				if ( preg_match( '/alt_image_/i', $key ) ) {
					$alt_images[] = $this->upload_attachment_to_product( $val, $id );
				}
				if ( $key == 'image_main' ) {
					$image_main = $this->upload_attachment_to_product( $val, $id );
				}
				if ( preg_match( '/image_gallery/i', $key ) ) {
					$image_gallery[] = $this->upload_attachment_to_product( $val, $id );
				}
			}
			
			// Save the alternate images
			if ( ! empty( $alt_images ) || $alt_images_empty == true ) {
				$existing_alt_images = get_post_meta( $id, apply_filters('in8_product_additional_variation_images', '_wc_additional_variation_images'), TRUE );
				if ( ! empty( $existing_alt_images ) ) {
					$existing_alt_images = explode( ',', $existing_alt_images );
					foreach ( $existing_alt_images as $img ) {
						wp_delete_attachment( $img, TRUE );
//						wp_schedule_single_event(time()+ 15, 'wp_delete_attachment', [$img, true]);
					}
				}
				
				// Save the new images
				update_post_meta( $id, apply_filters('in8_product_additional_variation_images', '_wc_additional_variation_images'), implode( ',', $alt_images ) );
			}
			
			// Save the main product image
			if ( ! empty( $image_main ) ) {
				$existing_image_main = get_post_thumbnail_id( $id );
				if ( ! empty( $existing_image_main ) ) {
					wp_delete_attachment( $existing_image_main, TRUE );
//					wp_schedule_single_event(time()+ 15, 'wp_delete_attachment', [$existing_image_main, true]);
				}
				
				if( $image_main != 'empty' ) {
					set_post_thumbnail( $id, $image_main );
				}
			}
			
			// Save the gallery images
			if ( ! empty( $image_gallery ) || $image_gallery_empty == true ) {
				$existing_gallery_images = get_post_meta( $id, '_product_image_gallery', TRUE );
				if ( ! empty( $existing_gallery_images ) ) {
					$existing_gallery_images = explode( ',', $existing_gallery_images );
					foreach ( $existing_gallery_images as $img ) {
						wp_delete_attachment( $img, TRUE );
//						wp_schedule_single_event(time()+ 15, 'wp_delete_attachment', [$img, true]);
					}
				}
				
				// Save the new images
				update_post_meta( $id, '_product_image_gallery', implode( ',', $image_gallery ) );
			}
			
			
			do_action( 'in8_endpoint_meta_post_after_update', $data, $id );
			echo json_encode( [ "success" => "true" ] );
			
			exit;
		}
		
		
		public function upload_attachment_to_product( $image_url, $product_id ) {
			$upload = $this->wc_rest_upload_image_from_url( $image_url );
			
			$result = $this->set_uploaded_image_as_attachment( $upload, $product_id );
			
			
			return $result;
		}
		
		public function wc_rest_upload_image_from_url( $image_url ) {
			$file_name  = basename( current( explode( '?', $image_url ) ) );
			
			// If file name is media.nl then use the filename parameter from the url to get the filename, if that's not present default the file name to image.jpg
			if( $file_name == 'media.nl' ){
				$parts = parse_url($image_url);
				parse_str($parts['query'], $query);
				if( !empty($query['filename']) ){
					$file_name = $query['filename'];
				}else{
					$file_name = 'image.jpg';
				}
			}
			$parsed_url = @parse_url( $image_url );
			
			// Check parsed URL.
			if ( ! $parsed_url || ! is_array( $parsed_url ) ) {
				return new WP_Error( 'woocommerce_rest_invalid_image_url', sprintf( __( 'Invalid URL %s.', 'woocommerce' ), $image_url ), [ 'status' => 400 ] );
			}
			
			// Ensure url is valid.
			$image_url = esc_url_raw( $image_url );
			
			// Get the file.
			$response = wp_safe_remote_get( $image_url, [
				'timeout' => 10,
			] );
			
			// messy trick to set the file name
			if( apply_filters('in8_save_image_messy_filename_enabled', false) ){
				$headersObject = $response['headers'];
				$headers = $headersObject->getAll();
				preg_match("/[^\'\']+$/", $headers['content-disposition'], $matches);
				if( !empty($matches[0]) ){
					$file_name = $matches[0];
				}
				$headersContentType = $headers['content-type'];
				if( preg_match('/jpg/i', $headersContentType) || preg_match('/jpeg/i', $headersContentType) ){
					$file_name = 'image.jpg';
				}
			}
			
			$file_name = apply_filters('in8_save_image_filename', $file_name, $image_url, $response);
			
			if ( is_wp_error( $response ) ) {
				return new WP_Error( 'woocommerce_rest_invalid_remote_image_url', sprintf( __( 'Error getting remote image %s.', 'woocommerce' ), $image_url ) . ' ' . sprintf( __( 'Error: %s.', 'woocommerce' ), $response->get_error_message() ), [ 'status' => 400 ] );
			} elseif ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
				return new WP_Error( 'woocommerce_rest_invalid_remote_image_url', sprintf( __( 'Error getting remote image %s.', 'woocommerce' ), $image_url ), [ 'status' => 400 ] );
			}
			
			
			// Upload the file.
			$upload = wp_upload_bits( $file_name, '', wp_remote_retrieve_body( $response ) );
			
			if ( $upload['error'] ) {
				return new WP_Error( 'woocommerce_rest_image_upload_error', $upload['error'], [ 'status' => 400 ] );
			}
			
			// Get filesize.
			$filesize = filesize( $upload['file'] );
			
			if ( 0 == $filesize ) {
				@unlink( $upload['file'] );
				unset( $upload );
				
				return new WP_Error( 'woocommerce_rest_image_upload_file_error', __( 'Zero size file downloaded.', 'woocommerce' ), [ 'status' => 400 ] );
			}
			
			do_action( 'woocommerce_rest_api_uploaded_image_from_url', $upload, $image_url );
			
			return $upload;
		}
		
		public function set_uploaded_image_as_attachment( $upload, $product_id = 0 ) {
			if( is_wp_error($upload) ){
				echo "<pre>";print_r('image failed to upload with error:');echo "</pre>";
				echo "<pre>";print_r($upload);echo "</pre>";
				return;
			}
			$info    = wp_check_filetype( $upload['file'] );
			$title   = '';
			$content = '';
			
			if ( $image_meta = @wp_read_image_metadata( $upload['file'] ) ) {
				if ( trim( $image_meta['title'] ) && ! is_numeric( sanitize_title( $image_meta['title'] ) ) ) {
					$title = wc_clean( $image_meta['title'] );
				}
				if ( trim( $image_meta['caption'] ) ) {
					$content = wc_clean( $image_meta['caption'] );
				}
			}
			
			$attachment = [
				'post_mime_type' => $info['type'],
				'guid'           => $upload['url'],
				'post_parent'    => $product_id,
				'post_title'     => $title,
				'post_content'   => $content,
			];
			
			$attachment_id = wp_insert_attachment( $attachment, $upload['file'], $product_id );
			if ( ! is_wp_error( $attachment_id ) ) {
				wp_update_attachment_metadata( $attachment_id, wp_generate_attachment_metadata( $attachment_id, $upload['file'] ) );
			}
			
			return $attachment_id;
		}
		
		
	}
    